/*! 
    \file sound_engine.h
    \brief Definition of the SoundEngine

    !!AVOID!!:
    SDL_OpenAudio
     Use Mix_OpenAudio instead. 
    SDL_CloseAudio
     Use Mix_CloseAudio instead. 
    SDL_PauseAudio
     Use Mix_Pause(-1) and Mix_PauseMusic instead, to pause.
     Use Mix_Resume(-1) and Mix_ResumeMusic instead, to unpause. 
    SDL_LockAudio
     This is just not needed since SDL_mixer handles this for you.
     Using it may cause problems as well. 
    SDL_UnlockAudio
     This is just not needed since SDL_mixer handles this for you.
     Using it may cause problems as well. 
*/

#ifndef _SOUND_ENGINE_H
#define _SOUND_ENGINE_H

#include "base_object.h"
#include <SDL_mixer.h>

#define SOUND_DEFAULT_FREQUENCY 44100
#define SOUND_DEFAULT_CHANNELS 2
#define SOUND_DEFAULT_BUFIZE 16384

// an enumerator of the different Sound Types.
typedef enum SOUND_TYPE {SOUND_NONE,
			 SOUND_WAV, /* .wav */
			 SOUND_MOD, /* .mod .s3m .it .xm */
			 SOUND_MID, /* mid */
			 SOUND_OGG,
			 SOUND_MP3};

// FORWARD DEFINITION

class Sound {
 public:
  Sound(const char* fileName, SOUND_TYPE soundType);
  virtual ~Sound(void);
 protected:
  SOUND_TYPE soundType;
  int channel;
};

class SoundEffect : public Sound {
 public:
  SoundEffect(const char* fileName, SOUND_TYPE soundType = SOUND_NONE);
  virtual ~SoundEffect(void);
 private:
  Mix_Chunk* effect;
};

class Music :public Sound {
 public:
  Music(const char* fileName, SOUND_TYPE soundType = SOUND_NONE);
  virtual ~Music(void);
 private:
  Mix_Music* music;
};

//! The Main Sound Engine
class SoundEngine : public BaseObject {

 public:
  static SoundEngine* getInstance(void);
  virtual ~SoundEngine(void);


  void setNumberOfChannels(unsigned int numberOfChannels);


  static bool isInit;
  static bool enabled;
 private:
  SoundEngine(void);
  static SoundEngine* singletonRef;

  void enableSound(void);
  void disableSound(void);
  static bool checkVersion(void);


  // settings
  int volume;
  unsigned int numberOfChannels;

  // audio specifications
  int frequency;
  Uint16 format;
  int channels;
  int bits;
  int buffers;



};


#endif /* _SOUND_ENGINE_H */
